cmd

package
v0.0.0-...-209b98f Latest Latest
Warning

This package is not in the latest version of its module.

Go to latest
Published: Feb 12, 2019 License: MIT Imports: 10 Imported by: 0

Documentation

Index

Constants

This section is empty.

Variables

This section is empty.

Functions

func GenAsm

func GenAsm(file string, dst string, runtime bool)

GenAsm generates the assembly code from the input go program and writes it into file dst.

func GenAsmFromIR

func GenAsmFromIR(file string, dst string, runtime bool)

GenAsmFromIR generates the assembly code using IR generated from the input program and writes it into file dst.

func GenHTML

func GenHTML(file string) error

GenHTML generates the rightmost derivations used in the bottom-up parsing and pretty-prints them in an HTML format.

func GenIR

func GenIR(file string)

GenIR generates the IR instructions from the input program.

func GenToken

func GenToken(file string)

GenToken generates the tokens returned by lexer from the input program.

Types

This section is empty.

Jump to