types

package
v0.0.0-...-209b98f Latest Latest
Warning

This package is not in the latest version of its module.

Go to latest
Published: Feb 12, 2019 License: MIT Imports: 0 Imported by: 0

Documentation

Overview

Package types defines type information used by the register allocator and the three-address code data structure. The usage of this package arises when type information collectd from the three-address code has to be propogated to the register allocator.

Index

Constants

This section is empty.

Variables

This section is empty.

Functions

This section is empty.

Types

type RegType

type RegType int

RegType determines the type of the variable stored by a register.

const (
	NIL RegType = iota
	INT
	STR
	ARR
)

Jump to