Documentation
¶
Index ¶
Constants ¶
This section is empty.
Variables ¶
This section is empty.
Functions ¶
func GenAsm ¶
GenAsm generates the assembly code from the input go program and writes it into file dst.
func GenAsmFromIR ¶
GenAsmFromIR generates the assembly code using IR generated from the input program and writes it into file dst.
Types ¶
This section is empty.
Click to show internal directories.
Click to hide internal directories.