Documentation
¶
Index ¶
Constants ¶
This section is empty.
Variables ¶
This section is empty.
Functions ¶
func AppendCode ¶
AppendCode is a variadic function which takes multiple strings as arguments and appends them to a slice of IR codes.
func SplitAndSanitize ¶
SplitAndSanitize creates a slice after splitting a string at a separator. The entries in resulting slice are trimmed of any whitespace and the resulting entries which are empty are removed (originally containing only whitspaces).
Types ¶
type Item ¶
type Item struct {
// contains filtered or unexported fields
}
Item defines a single element of the stack.
Click to show internal directories.
Click to hide internal directories.